<?php

/**
 * This class contains common functions for creating invoices and orders
 *
 * @package   WooCommerce_Ready_To_Order_Integration
 * @author    BjornTech <hello@bjorntech.com>
 * @license   GPL-3.0
 * @link      http://bjorntech.com
 * @copyright 2017-2020 BjornTech
 */

defined('ABSPATH') || exit;

if (!class_exists('Wr2o_WooCommerce_Document_Admin', false)) {

    class Wr2o_WooCommerce_Document_Admin
    {

        public function __construct()
        {

            add_filter('manage_edit-shop_order_columns', array($this, 'document_number_header'), 20);
            add_action('manage_shop_order_posts_custom_column', array($this, 'invoice_number_content'));
            add_filter('handle_bulk_actions-edit-shop_order', array($this, 'handle_bulk_actions'), 10, 3);
            add_filter('bulk_actions-edit-shop_order', array($this, 'define_bulk_actions'));

        }

        public function order_meta_general($order)
        {

            $ready2order_id = $order->get_id() ? Wr2o_Helper::get_ready2order_invoice_number($order->get_id()) : '' ;
            echo '<br class="clear" />';
            echo '<h4>ready2order invoice</h4>';
            echo '<div class="address">';
            echo '<p>' . $ready2order_id . '</p>';
            echo '</div>';

        }

        public function document_number_header($columns)
        {

            $creates = get_option('wr2o_enable_invoice_export') == 'yes';

            $new_columns = array();

            foreach ($columns as $column_name => $column_info) {

                $new_columns[$column_name] = $column_info;

                if ('order_number' === $column_name) {

                    if($creates){
                        $new_columns['ready2order_invoice_number'] = __('ready2order invoice', 'woo-fortnox-hub');
                    }
                }
            }

            return $new_columns;

        }

        public function invoice_number_content($column)
        {
            global $post;

            if ('ready2order_invoice_number' == $column) {
                $wr2o_invoice = Wr2o_Helper::get_ready2order_full_invoice_number($post->ID);

                echo sprintf('%s', $wr2o_invoice ? $wr2o_invoice : '-');
            }

        }

        public function handle_bulk_actions($redirect_to, $action, $ids)
        {
            if ('ready2order_sync_invoice' == $action) {
                foreach (array_reverse($ids) as $order_id) {
                    if (is_admin() && ('yes' !== get_option('wr2o_queue_admin_requests'))) { 
                        do_action('w2ro_processing_invoice',$order_id);
                    } else {
                        as_schedule_single_action(as_get_datetime_object(), 'w2ro_processing_invoice', array($order_id));
                    }

                }
            }

            if ('ready2order_clean_invoice' == $action) {
                foreach (array_reverse($ids) as $order_id) {
                    if (is_admin() && ('yes' !== get_option('wr2o_queue_admin_requests'))) { 
                        do_action('w2ro_clean_invoice',$order_id);
                    } else {
                        as_schedule_single_action(as_get_datetime_object(), 'w2ro_clean_invoice', array($order_id));
                    }
                }
            }

            return esc_url_raw($redirect_to);
        }

        public function define_bulk_actions($actions)
        {
            if (get_option('wr2o_enable_invoice_export') == 'yes') {
                $actions['ready2order_sync_invoice'] = __('Create ready2order invoice from WooCommerce order', 'woo-fortnox-hub');
                $actions['ready2order_clean_invoice'] = __('Clean ready2order invoice from WooCommerce order', 'woo-fortnox-hub');
            }
            return $actions;
        }

    }

    new Wr2o_WooCommerce_Document_Admin();
}
